Package: peakPantheR
Title: Peak Picking and Annotation of High Resolution Experiments
Version: 1.27.0
Date: 2025-09-27
Authors@R: c(person("Arnaud", "Wolfer", email = "adwolfer@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0001-5856-3218")),
	person("Goncalo", "Correia", email = "gscorreia89@gmail.com", role = "aut", comment = c(ORCID = "0000-0001-8271-9294")),
	person("Jake", "Pearce", email = "jake.pearce@gmail.com", role = "ctb"),
	person("Caroline", "Sands", email = "caz119@hotmail.co.uk", role = "ctb"))
Description: An automated pipeline for the detection, integration and reporting of predefined features across a large number of mass spectrometry data files.
    It enables the real time annotation of multiple compounds in a single file, or the parallel annotation of multiple compounds in multiple files.
    A graphical user interface as well as command line functions will assist in assessing the quality of annotation and update fitting parameters until a satisfactory result is obtained.
Depends: R (>= 4.5)
Imports: foreach (>= 1.4.4), doParallel (>= 1.0.11), ggplot2 (>=
        3.5.0), gridExtra (>= 2.3), MSnbase (>= 2.4.0), mzR (>=
        2.12.0), stringr (>= 1.2.0), methods (>= 3.4.0), XML (>=
        3.98.1.10), minpack.lm (>= 1.2.1), scales(>= 0.5.0), shiny (>=
        1.0.5), bslib, shinycssloaders (>= 1.0.0), DT (>= 0.15), pracma
        (>= 2.2.3), utils, lubridate, svglite (>= 2.1.1)
biocViews: MassSpectrometry, Metabolomics, PeakDetection
License: GPL-3
BugReports: https://github.com/phenomecentre/peakPantheR/issues/new
URL: https://github.com/phenomecentre/peakPantheR
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: testthat, devtools, faahKO, msdata, knitr, rmarkdown, pander,
        BiocStyle
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/peakPantheR
git_branch: devel
git_last_commit: efe4373
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-07-14
NeedsCompilation: no
Packaged: 2026-07-14 23:30:03 UTC; biocbuild
Author: Arnaud Wolfer [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5856-3218>),
  Goncalo Correia [aut] (ORCID: <https://orcid.org/0000-0001-8271-9294>),
  Jake Pearce [ctb],
  Caroline Sands [ctb]
Maintainer: Arnaud Wolfer <adwolfer@gmail.com>
