# Generated by roxygen2: do not edit by hand

export(annotation_diagnostic_multiplot_UI_helper)
export(annotation_fit_summary_UI_helper)
export(annotation_showMethod_UI_helper)
export(annotation_showText_UI_helper)
export(initialise_annotation_from_files_UI_helper)
export(load_annotation_from_file_UI_helper)
export(outputAnnotationFeatureMetadata_UI_helper)
export(outputAnnotationSpectraMetadata_UI_helper)
export(peakPantheRAnnotation)
export(peakPantheR_ROIStatistics)
export(peakPantheR_loadAnnotationParamsCSV)
export(peakPantheR_parallelAnnotation)
export(peakPantheR_plotEICFit)
export(peakPantheR_plotPeakwidth)
export(peakPantheR_quickEIC)
export(peakPantheR_singleFileSearch)
export(peakPantheR_start_GUI)
export(spectraPaths_and_metadata_UI_helper)
export(spectra_metadata_colourScheme_UI_helper)
exportMethods(EICs)
exportMethods(FIR)
exportMethods(ROI)
exportMethods(TIC)
exportMethods(acquisitionTime)
exportMethods(annotationDiagnosticPlots)
exportMethods(annotationParamsDiagnostic)
exportMethods(annotationTable)
exportMethods(cpdID)
exportMethods(cpdMetadata)
exportMethods(cpdName)
exportMethods(dataPoints)
exportMethods(filename)
exportMethods(filepath)
exportMethods(isAnnotated)
exportMethods(nbCompounds)
exportMethods(nbSamples)
exportMethods(outputAnnotationDiagnostic)
exportMethods(outputAnnotationParamsCSV)
exportMethods(outputAnnotationResult)
exportMethods(peakFit)
exportMethods(peakTables)
exportMethods(resetAnnotation)
exportMethods(resetFIR)
exportMethods(retentionTimeCorrection)
exportMethods(spectraMetadata)
exportMethods(uROI)
exportMethods(uROIExist)
exportMethods(useFIR)
exportMethods(useUROI)
import(XML)
import(doParallel)
import(foreach)
import(methods)
import(mzR)
import(scales)
import(shiny)
import(shinycssloaders)
import(svglite)
import(utils)
importFrom(DT,DTOutput)
importFrom(DT,datatable)
importFrom(DT,renderDT)
importFrom(bslib,bs_theme)
