# 1. Generation of test_fakemzML.mzML
#
# The test_fakemzML.mzML was generated by converting a
# Waters .RAW file from an LC-MS experiment to the .mzML format,
# and then manually removing part of the <scan> tags containing raw data.
# Only the <mzML> and <run> sections, 3 example scans
# and the total ion current readout were kept. In this way the
# correct .mzML file structure is preserved for use in unit testing,
# but the file size is greatly decreased.
# The .RAW file was converted to .mzML using
# the ProteoWizard project msconvert tool.
