% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/curveFit_utils.R
\name{skewedGaussian_guess}
\alias{skewedGaussian_guess}
\title{Guess function for initial skewed gaussian parameters and bounds}
\usage{
skewedGaussian_guess(x, y)
}
\arguments{
\item{x}{(numeric) x values (e.g. retention time)}

\item{y}{(numeric) y observed values (e.g. spectra intensity)}
}
\value{
A list of guessed starting parameters \code{list()$init_params},
lower \code{list()$lower_bounds} and upper bounds \code{list()$upper_bounds}
(\code{$gamma}, \code{$center}, \code{$sigma}, \code{$amplitude})
}
\description{
Guess function for initial skewed gaussian parameters and
bounds, at the moment only checks the x position
}
